@@ -1,13 +1,15 @@
 # SPDX-FileCopyrightText: 2017 Free Software Foundation Europe e.V. <https://fsfe.org>
-# SPDX-FileCopyrightText: 2022 Florian Snow <florian@familysnow.net>
 # SPDX-FileCopyrightText: 2022 Carmen Bianca Bakker <carmenbianca@fsfe.org>
+# SPDX-FileCopyrightText: 2022 Florian Snow <florian@familysnow.net>
+# SPDX-FileCopyrightText: 2023 Matthias Riße
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 """Global fixtures and configuration."""
 
-# pylint: disable=redefined-outer-name
+# pylint: disable=redefined-outer-name,invalid-name
 
+import contextlib
 import datetime
 import logging
 import multiprocessing as mp
@@ -22,7 +24,6 @@ from typing import Generator
 from unittest.mock import create_autospec
 
 import pytest
-from debian.copyright import Copyright
 from jinja2 import Environment
 
 os.environ["LC_ALL"] = "C"
@@ -36,13 +37,30 @@ try:
 except ImportError:
     sys.path.append(os.path.join(Path(__file__).parent.parent, "src"))
 finally:
-    from reuse._util import GIT_EXE, HG_EXE, setup_logging
+    from reuse._util import GIT_EXE, HG_EXE, PIJUL_EXE, setup_logging
+    from reuse.global_licensing import ReuseDep5
 
 CWD = Path.cwd()
 
 TESTS_DIRECTORY = Path(__file__).parent.resolve()
 RESOURCES_DIRECTORY = TESTS_DIRECTORY / "resources"
 
+try:
+    import pwd
+
+    is_root = pwd.getpwuid(os.getuid()).pw_name == "root"
+    is_posix = True
+except ImportError:
+    is_root = False
+    is_posix = False
+
+cpython = pytest.mark.skipif(
+    sys.implementation.name != "cpython", reason="only CPython supported"
+)
+git = pytest.mark.skipif(not GIT_EXE, reason="requires git")
+no_root = pytest.mark.xfail(is_root, reason="fails when user is root")
+posix = pytest.mark.skipif(not is_posix, reason="Windows not supported")
+
 
 # REUSE-IgnoreStart
 
@@ -66,6 +84,13 @@ def pytest_runtest_setup(item):
     # Make sure to restore CWD
     os.chdir(CWD)
 
+    # TODO: Awful workaround. In `main`, this environment variable is set under
+    # certain conditions. This means that all tests that run _after_ that
+    # condition is met also have the environment variable set, because the
+    # environment had been changed. There should be a better way to scope this.
+    with contextlib.suppress(KeyError):
+        del os.environ["_SUPPRESS_DEP5_WARNING"]
+
 
 @pytest.fixture()
 def git_exe() -> str:
@@ -83,6 +108,14 @@ def hg_exe() -> str:
     return str(HG_EXE)
 
 
+@pytest.fixture()
+def pijul_exe() -> str:
+    """Run the test with Pijul."""
+    if not PIJUL_EXE:
+        pytest.skip("cannot run this test without pijul")
+    return str(PIJUL_EXE)
+
+
 @pytest.fixture(params=[True, False])
 def multiprocessing(request, monkeypatch) -> Generator[bool, None, None]:
     """Run the test with or without multiprocessing."""
@@ -141,6 +174,25 @@ def fake_repository(tmpdir_factory) -> Path:
     return directory
 
 
+@pytest.fixture()
+def fake_repository_reuse_toml(fake_repository) -> Path:
+    """Add REUSE.toml to the fake repo."""
+    shutil.copy(
+        RESOURCES_DIRECTORY / "REUSE.toml", fake_repository / "REUSE.toml"
+    )
+    (fake_repository / "doc/index.rst").touch()
+    return fake_repository
+
+
+@pytest.fixture()
+def fake_repository_dep5(fake_repository) -> Path:
+    """Add .reuse/dep5 to the fake repo."""
+    (fake_repository / ".reuse").mkdir(exist_ok=True)
+    shutil.copy(RESOURCES_DIRECTORY / "dep5", fake_repository / ".reuse/dep5")
+    (fake_repository / "doc/index.rst").touch()
+    return fake_repository
+
+
 def _repo_contents(
     fake_repository, ignore_filename=".gitignore", ignore_prefix=""
 ):
@@ -224,8 +276,33 @@ def hg_repository(fake_repository: Path, hg_exe: str) -> Path:
 
 
 @pytest.fixture()
+def pijul_repository(fake_repository: Path, pijul_exe: str) -> Path:
+    """Create a pijul repository with ignored files."""
+    os.chdir(fake_repository)
+    _repo_contents(
+        fake_repository,
+        ignore_filename=".ignore",
+    )
+
+    subprocess.run([pijul_exe, "init", "."], check=True)
+    subprocess.run([pijul_exe, "add", "--recursive", "."], check=True)
+    subprocess.run(
+        [
+            pijul_exe,
+            "record",
+            "--all",
+            "--message",
+            "initial",
+        ],
+        check=True,
+    )
+
+    return fake_repository
+
+
+@pytest.fixture(params=["submodule-add", "manual"])
 def submodule_repository(
-    git_repository: Path, git_exe: str, tmpdir_factory
+    git_repository: Path, git_exe: str, tmpdir_factory, request
 ) -> Path:
     """Create a git repository that contains a submodule."""
     header = cleandoc(
@@ -260,45 +337,62 @@ def submodule_repository(
 
     os.chdir(git_repository)
 
-    subprocess.run(
-        [
-            git_exe,
-            # https://git-scm.com/docs/git-config#Documentation/git-config.txt-protocolallow
-            #
-            # This circumvents a bug/behaviour caused by CVE-2022-39253 where
-            # you cannot use `git submodule add repository path` where
-            # repository is a file on the filesystem.
-            "-c",
-            "protocol.file.allow=always",
-            "submodule",
-            "add",
-            str(submodule.resolve()),
-            "submodule",
-        ],
-        check=True,
-    )
-    subprocess.run(
-        [
-            git_exe,
-            "commit",
-            "-m",
-            "add submodule",
-        ],
-        check=True,
-    )
+    if request.param == "submodule-add":
+        subprocess.run(
+            [
+                git_exe,
+                # https://git-scm.com/docs/git-config#Documentation/git-config.txt-protocolallow
+                #
+                # This circumvents a bug/behaviour caused by CVE-2022-39253
+                # where you cannot use `git submodule add repository path` where
+                # repository is a file on the filesystem.
+                "-c",
+                "protocol.file.allow=always",
+                "submodule",
+                "add",
+                str(submodule.resolve()),
+                "submodule",
+            ],
+            check=True,
+        )
+    elif request.param == "manual":
+        subprocess.run(
+            [git_exe, "clone", str(submodule.resolve()), "submodule"],
+            check=True,
+        )
+        with open(
+            git_repository / ".gitmodules", mode="a", encoding="utf-8"
+        ) as gitmodules_file:
+            gitmodules_file.write(
+                f"""[submodule "submodule"]
+	path = submodule
+	url = {submodule.resolve().as_posix()}
+"""
+            )
+        subprocess.run(
+            [
+                git_exe,
+                "add",
+                "--no-warn-embedded-repo",
+                ".gitmodules",
+                "submodule",
+            ],
+            check=True,
+        )
 
+    subprocess.run(
+        [git_exe, "commit", "-m", "add submodule"],
+        check=True,
+    )
     (git_repository / ".gitmodules.license").write_text(header)
 
     return git_repository
 
 
 @pytest.fixture(scope="session")
-def dep5_copyright():
-    """Create a dep5 Copyright object."""
-    with (RESOURCES_DIRECTORY / "fake_repository/.reuse/dep5").open(
-        encoding="utf-8"
-    ) as fp:
-        return Copyright(fp)
+def reuse_dep5():
+    """Create a ReuseDep5 object."""
+    return ReuseDep5.from_file(RESOURCES_DIRECTORY / "dep5")
 
 
 @pytest.fixture()
