@@ -3,6 +3,7 @@
 # SPDX-FileCopyrightText: © 2020 Liferay, Inc. <https://liferay.com>
 # SPDX-FileCopyrightText: 2022 Florian Snow <florian@familysnow.net>
 # SPDX-FileCopyrightText: 2022 Pietro Albini <pietro.albini@ferrous-systems.com>
+# SPDX-FileCopyrightText: 2024 Carmen Bianca BAKKER <carmenbianca@fsfe.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -14,6 +15,8 @@ import errno
 import json
 import os
 import re
+import shutil
+import warnings
 from inspect import cleandoc
 from pathlib import Path
 from typing import Generator, Optional
@@ -21,10 +24,12 @@ from unittest.mock import create_autospec
 from urllib.error import URLError
 
 import pytest
+from conftest import RESOURCES_DIRECTORY
+from freezegun import freeze_time
 
 from reuse import download
 from reuse._main import main
-from reuse._util import GIT_EXE, HG_EXE
+from reuse._util import GIT_EXE, HG_EXE, PIJUL_EXE, cleandoc_nl
 from reuse.report import LINT_VERSION
 
 # REUSE-IgnoreStart
@@ -36,7 +41,7 @@ def optional_git_exe(
 ) -> Generator[Optional[str], None, None]:
     """Run the test with or without git."""
     exe = GIT_EXE if request.param else ""
-    monkeypatch.setattr("reuse.project.GIT_EXE", exe)
+    monkeypatch.setattr("reuse.vcs.GIT_EXE", exe)
     monkeypatch.setattr("reuse._util.GIT_EXE", exe)
     yield exe
 
@@ -47,11 +52,22 @@ def optional_hg_exe(
 ) -> Generator[Optional[str], None, None]:
     """Run the test with or without mercurial."""
     exe = HG_EXE if request.param else ""
-    monkeypatch.setattr("reuse.project.HG_EXE", exe)
+    monkeypatch.setattr("reuse.vcs.HG_EXE", exe)
     monkeypatch.setattr("reuse._util.HG_EXE", exe)
     yield exe
 
 
+@pytest.fixture(params=[True, False])
+def optional_pijul_exe(
+    request, monkeypatch
+) -> Generator[Optional[str], None, None]:
+    """Run the test with or without Pijul."""
+    exe = PIJUL_EXE if request.param else ""
+    monkeypatch.setattr("reuse.vcs.PIJUL_EXE", exe)
+    monkeypatch.setattr("reuse._util.PIJUL_EXE", exe)
+    yield exe
+
+
 @pytest.fixture()
 def mock_put_license_in_file(monkeypatch):
     """Create a mocked version of put_license_in_file."""
@@ -70,6 +86,22 @@ def test_lint(fake_repository, stringio, optional_git_exe, optional_hg_exe):
     assert ":-)" in stringio.getvalue()
 
 
+def test_lint_reuse_toml(fake_repository_reuse_toml, stringio):
+    """Run a simple lint with REUSE.toml."""
+    result = main(["lint"], out=stringio)
+
+    assert result == 0
+    assert ":-)" in stringio.getvalue()
+
+
+def test_lint_dep5(fake_repository_dep5, stringio):
+    """Run a simple lint with .reuse/dep5."""
+    result = main(["lint"], out=stringio)
+
+    assert result == 0
+    assert ":-)" in stringio.getvalue()
+
+
 def test_lint_git(git_repository, stringio):
     """Run a successful lint."""
     result = main(["lint"], out=stringio)
@@ -221,6 +253,24 @@ def test_lint_fail_quiet(fake_repository, stringio):
     assert stringio.getvalue() == ""
 
 
+def test_lint_dep5_decode_error(fake_repository_dep5, capsys):
+    """Display an error if dep5 cannot be decoded."""
+    shutil.copy(
+        RESOURCES_DIRECTORY / "fsfe.png", fake_repository_dep5 / ".reuse/dep5"
+    )
+    with pytest.raises(SystemExit):
+        main(["lint"])
+    assert "could not be decoded" in capsys.readouterr().err
+
+
+def test_lint_dep5_parse_error(fake_repository_dep5, capsys):
+    """Display an error if there's a dep5 parse error."""
+    (fake_repository_dep5 / ".reuse/dep5").write_text("foo")
+    with pytest.raises(SystemExit):
+        main(["lint"])
+    assert "could not be parsed" in capsys.readouterr().err
+
+
 def test_lint_json(fake_repository, stringio):
     """Run a failed lint."""
     result = main(["lint", "--json"], out=stringio)
@@ -261,7 +311,7 @@ def test_lint_custom_root(fake_repository, stringio):
     result = main(["--root", "doc", "lint"], out=stringio)
 
     assert result > 0
-    assert "index.rst" in stringio.getvalue()
+    assert "usage.md" in stringio.getvalue()
     assert ":-(" in stringio.getvalue()
 
 
@@ -270,14 +320,16 @@ def test_lint_custom_root_git(git_repository, stringio):
     result = main(["--root", "doc", "lint"], out=stringio)
 
     assert result > 0
-    assert "index.rst" in stringio.getvalue()
+    assert "usage.md" in stringio.getvalue()
     assert ":-(" in stringio.getvalue()
 
 
-def test_lint_custom_root_different_cwd(fake_repository, stringio):
+def test_lint_custom_root_different_cwd(fake_repository_reuse_toml, stringio):
     """Use a custom root while CWD is different."""
     os.chdir("/")
-    result = main(["--root", str(fake_repository), "lint"], out=stringio)
+    result = main(
+        ["--root", str(fake_repository_reuse_toml), "lint"], out=stringio
+    )
 
     assert result == 0
     assert ":-)" in stringio.getvalue()
@@ -303,6 +355,7 @@ def test_lint_no_multiprocessing(fake_repository, stringio, multiprocessing):
     assert ":-)" in stringio.getvalue()
 
 
+@freeze_time("2024-04-08T17:34:00Z")
 def test_spdx(fake_repository, stringio):
     """Compile to an SPDX document."""
     os.chdir(str(fake_repository))
@@ -314,6 +367,7 @@ def test_spdx(fake_repository, stringio):
     assert "\nLicenseConcluded: GPL-3.0-or-later\n" not in output
     assert "\nCreator: Person: Anonymous ()\n" in output
     assert "\nCreator: Organization: Anonymous ()\n" in output
+    assert "\nCreated: 2024-04-08T17:34:00Z\n" in output
 
     # TODO: This test is rubbish.
     assert result == 0
@@ -385,7 +439,7 @@ def test_download(fake_repository, stringio, mock_put_license_in_file):
 
     assert result == 0
     mock_put_license_in_file.assert_called_with(
-        "0BSD", Path("LICENSES/0BSD.txt").resolve()
+        "0BSD", Path("LICENSES/0BSD.txt").resolve(), source=None
     )
 
 
@@ -434,7 +488,9 @@ def test_download_custom_output(
     result = main(["download", "-o", "foo", "0BSD"], out=stringio)
 
     assert result == 0
-    mock_put_license_in_file.assert_called_with("0BSD", destination=Path("foo"))
+    mock_put_license_in_file.assert_called_with(
+        "0BSD", destination=Path("foo"), source=None
+    )
 
 
 def test_download_custom_output_too_many(
@@ -449,6 +505,102 @@ def test_download_custom_output_too_many(
         )
 
 
+def test_download_inside_licenses_dir(
+    fake_repository, stringio, mock_put_license_in_file
+):
+    """While inside the LICENSES/ directory, don't create another LICENSES/
+    directory.
+    """
+    os.chdir(fake_repository / "LICENSES")
+    result = main(["download", "0BSD"], out=stringio)
+    assert result == 0
+    mock_put_license_in_file.assert_called_with(
+        "0BSD", destination=Path("0BSD.txt").absolute(), source=None
+    )
+
+
+def test_download_inside_licenses_dir_in_git(
+    git_repository, stringio, mock_put_license_in_file
+):
+    """While inside a random LICENSES/ directory in a Git repository,.use the
+    root LICENSES/ directory.
+    """
+    (git_repository / "doc/LICENSES").mkdir()
+    os.chdir(git_repository / "doc/LICENSES")
+    result = main(["download", "0BSD"], out=stringio)
+    assert result == 0
+    mock_put_license_in_file.assert_called_with(
+        "0BSD", destination=Path("../../LICENSES/0BSD.txt"), source=None
+    )
+
+
+def test_download_different_root(
+    fake_repository, stringio, mock_put_license_in_file
+):
+    """Download using a different root."""
+    (fake_repository / "new_root").mkdir()
+
+    result = main(
+        [
+            "--root",
+            str((fake_repository / "new_root").resolve()),
+            "download",
+            "MIT",
+        ],
+        out=stringio,
+    )
+    assert result == 0
+    mock_put_license_in_file.assert_called_with(
+        "MIT", Path("new_root/LICENSES/MIT.txt").resolve(), source=None
+    )
+
+
+def test_download_licenseref_no_source(empty_directory, stringio):
+    """Downloading a LicenseRef license creates an empty file."""
+    main(["download", "LicenseRef-hello"], out=stringio)
+    assert (empty_directory / "LICENSES/LicenseRef-hello.txt").read_text() == ""
+
+
+def test_download_licenseref_source_file(empty_directory, stringio):
+    """Downloading a LicenseRef license with a source file copies that file's
+    contents.
+    """
+    (empty_directory / "foo.txt").write_text("foo")
+    main(["download", "--source", "foo.txt", "LicenseRef-hello"], out=stringio)
+    assert (
+        empty_directory / "LICENSES/LicenseRef-hello.txt"
+    ).read_text() == "foo"
+
+
+def test_download_licenseref_source_dir(empty_directory, stringio):
+    """Downloading a LicenseRef license with a source dir copies the text from
+    the corresponding file in the directory.
+    """
+    (empty_directory / "lics").mkdir()
+    (empty_directory / "lics/LicenseRef-hello.txt").write_text("foo")
+
+    main(["download", "--source", "lics", "LicenseRef-hello"], out=stringio)
+    assert (
+        empty_directory / "LICENSES/LicenseRef-hello.txt"
+    ).read_text() == "foo"
+
+
+def test_download_licenseref_false_source_dir(empty_directory, stringio):
+    """Downloading a LicenseRef license with a source that does not contain the
+    license results in an error.
+    """
+    (empty_directory / "lics").mkdir()
+
+    result = main(
+        ["download", "--source", "lics", "LicenseRef-hello"], out=stringio
+    )
+    assert result != 0
+    assert (
+        f"{Path('lics') / 'LicenseRef-hello.txt'} does not exist"
+        in stringio.getvalue()
+    )
+
+
 def test_supported_licenses(stringio):
     """Invoke the supported-licenses command and check whether the result
     contains at least one license in the expected format.
@@ -462,4 +614,38 @@ def test_supported_licenses(stringio):
     )
 
 
+def test_convert_dep5(fake_repository_dep5, stringio):
+    """Convert a DEP5 repository to a REUSE.toml repository."""
+    result = main(["convert-dep5"], out=stringio)
+
+    assert result == 0
+    assert not (fake_repository_dep5 / ".reuse/dep5").exists()
+    assert (fake_repository_dep5 / "REUSE.toml").exists()
+    assert (fake_repository_dep5 / "REUSE.toml").read_text() == cleandoc_nl(
+        """
+        version = 1
+
+        [[annotations]]
+        path = "doc/**"
+        precedence = "aggregate"
+        SPDX-FileCopyrightText = "2017 Jane Doe"
+        SPDX-License-Identifier = "CC0-1.0"
+        """
+    )
+
+
+def test_convert_dep5_no_dep5_file(fake_repository, stringio):
+    """Cannot convert when there is no .reuse/dep5 file."""
+    with pytest.raises(SystemExit):
+        main(["convert-dep5"], out=stringio)
+
+
+def test_convert_dep5_no_warning(fake_repository_dep5, stringio):
+    """No PendingDeprecationWarning when running convert-dep5."""
+    with warnings.catch_warnings(record=True) as caught_warnings:
+        result = main(["convert-dep5"], out=stringio)
+        assert result == 0
+        assert not caught_warnings
+
+
 # REUSE-IgnoreEnd
